import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import matplotlib.dates as mdates
import datetime
from pandas.plotting import register_matplotlib_converters
register_matplotlib_converters()
import matplotlib.ticker as tick
import locale
locale.setlocale(locale.LC_ALL, 'fr_FR.UTF-8')#dates en français

#Extraction des données depuis le fichier .csv :
soleil=pd.read_csv('SoleilLune2022.csv',
sep=';', 
usecols=[1, 2, 4, 6], 
names=['date', 'lever', 'midi', 'coucher'], 
parse_dates=['date'], 
skiprows=1, 
nrows=365, 
)

#Extraction des listes de données pour les graphes :
soleil=soleil.set_index('date')
dates=soleil.index
lever=soleil['lever']
coucher=soleil['coucher']
midi=soleil['midi']
#EDT=midi-12#calcul de l'équation du temps

soleilHeureLegale=soleil
leverHL=[]
coucherHL=[]
midiHL=[]

heureEte=datetime.datetime(2022, 3, 27)
heureHiver=datetime.datetime(2022, 10, 30)

for N, jour in enumerate(dates):#ajoute encore 1 h pour heure d'été (TU+2)
    if jour>=heureEte and jour <heureHiver:
        leverHL.append(lever[N]+2)
        coucherHL.append(coucher[N]+2)
        midiHL.append(midi[N]+2)
    else:
        leverHL.append(lever[N]+1)
        coucherHL.append(coucher[N]+1)
        midiHL.append(midi[N]+1)

plt.rcParams["font.family"] = "Ubuntu"
plt.rcParams["font.size"] = 9

def HHmm(heure, lever):#écrit en base 60
    H=int(heure)
    min=round((heure-H)*60)
    if min<10:
        min="0"+str(min)
    return str(H)+'h'+str(min)[:-2]

#Paramètres du graphique :
fig=plt.figure(figsize=(29.7/2.54, 21/2.54), tight_layout=True)

# %%%%%%%%%%%%%%% Graphe 1
ax1=plt.subplot(221)
plt.yticks(np.arange(4, 24, 2), [str(N) +' h' for N in np.arange(4, 24, 2)])
ax1.set_title("Effet du changement d'heure saisonnier actuel (latitude 49.5° - Le Havre)")
ax1.grid()

#Mise en forme des graduations de dates en x :
locator1 = mdates.AutoDateLocator(minticks=10, maxticks=12)
formatter = mdates.ConciseDateFormatter(locator1)
ax1.xaxis.set_major_locator(locator1)
ax1.xaxis.set_major_formatter(formatter)
ax1.set_xlim(dates[0],dates[-1])

ax1.plot(dates, coucherHL)
ax1.plot(dates, leverHL)
ax1.set_ylim(3, 23)
plt.fill_between(dates, 0, leverHL, facecolor='royalblue', alpha = 0.3)
plt.fill_between(dates, coucherHL, 24, facecolor='royalblue', alpha = 0.3)

# %%%%%%%%%%%%%%% Graphe 2
ax2=plt.subplot(223)
plt.yticks(np.arange(4, 24, 2), [str(N) +' h' for N in np.arange(4, 24, 2)])
ax2.set_title("Maintien de l'heure dite 'd'hiver'")
ax2.grid()
ax2.xaxis.set_major_locator(locator1)
ax2.xaxis.set_major_formatter(formatter)
ax2.set_xlim(dates[0],dates[-1])

ax2.plot(dates, coucher+1)
ax2.plot(dates, lever+1)
ax2.set_ylim(3, 23)
plt.fill_between(dates, 0, lever+1, facecolor='royalblue', alpha = 0.3)
plt.fill_between(dates, coucher+1, 24, facecolor='royalblue', alpha = 0.3)

# %%%%%%%%%%%%%%% Graphe 3
ax3=plt.subplot(224)
plt.yticks(np.arange(4, 24, 2), [str(N) +' h' for N in np.arange(4, 24, 2)])
ax3.set_title("Maintien de l'heure dite 'd'été'")
ax3.grid()
ax3.xaxis.set_major_locator(locator1)
ax3.xaxis.set_major_formatter(formatter)
ax3.set_xlim(dates[0],dates[-1])
ax3.set_ylim(3, 23)

ax3.plot(dates, coucher+2)
ax3.plot(dates, lever+2)
plt.fill_between(dates, 0, lever+2, facecolor='royalblue', alpha = 0.3)
plt.fill_between(dates, coucher+2, 24, facecolor='royalblue', alpha = 0.3)

# %%%%%%%%%%%%%%% Texte
ax4=plt.subplot(222)
ax4.set_xticks([])
ax4.set_yticks([])

ax4.text(0.01,0.98,
         r"\begin{minipage}{12.8cm}"
         r"~~Le changement d'heure saisonnier a pour effet de décaler d'une heure les heures de lever"
         r"~et de coucher du Soleil. Contrairement à une idée répandue, \textit{il n'a aucun effet sur la durée du jour}."
         r"~La soirée estivale est rallongée d'une heure, mais l'heure de lever du Soleil est décalée dans le même sens."
         r"~Ce décalage du lever est peu perçu, car très matinal.\par "
         r"~~Si le choix est fait du maintien de l'heure appelée actuellement \textit{heure d'hiver}, une situation nouvelle se produirait l'été : le Soleil se lèverait et se coucherait 1~h plus tôt en été."
         r" Si au contraire c'est l'heure dite \textit{d'été} qui est maintenue, c'est en hiver que la situation serait nouvelle :"
         r"~le Soleil se lèverait et se coucherait 1~heure plus tard.\par ~~L'heure d'été est la plus populaire, en raison d'une confusion : elle est perçue comme responsable de la longue durée du jour estival ; de même l'heure d'hiver est accusée d'être à l'origine des longues nuits d'hiver."
         r" Pourtant, l'heure d'été nous décale davantage de l'heure solaire (2 h de décalage au lieu de 1 heure). En la choisissant comme définitive, nous conservons une heure de soleil supplémentaire les soirs d'été, mais perdons une heure de soleil matinal en hiver. \par ~~Les courbes données ici sont pour une latitude du nord de la France, mais les horaires resteraient comparables en France méridionale."
         r"~Les extrêmes des heures de lever et de coucher seraient légèrement moins marqués en allant vers le sud."
         r"\end{minipage}",
         transform=ax4.transAxes,
         usetex=True,fontsize=10.2,
         va='top')

fig.text(0.98,0.02,"Les calculs de visibilité des astres ont été réalisés par le service de calcul des éphémérides de l’IMCCE à travers son portail Système solaire (https://ssp.imcce.fr) ",
         fontsize=7,transform=fig.transFigure,ha='left',va='bottom', rotation=90)

fig.savefig("ChangementHeure.png", dpi=300)
fig.savefig("ChangementHeure.pdf",dpi=300)
plt.show()
